using System;
using System.Collections;
using System.Xml;
using System.IO;
using System.Net;
using System.Diagnostics;

namespace PageBox
{
	/// <summary>
	/// <p>Describes deployed archives.<br/>
	/// Serialized in pbArchives.xml.</p>
	/// <p>Copyright (c) 2002 Alexis Grandemange<br/>
	/// Mail: alexis.grandemange@pagebox.net</p>
	/// <pre>This program is free software; you can redistribute it and/or
	/// modify it under the terms of the GNU Lesser General Public
	/// License as published by the Free Software Foundation; version
	/// 2.1 of the License.
	/// This library is distributed in the hope that it will be useful,
	/// but WITHOUT ANY WARRANTY; without even the implied warranty of
	/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	/// GNU Lesser General Public License for more details.
	/// A copy of the GNU Lesser General Public License lesser.txt should be
	/// included in the distribution.</pre>
	/// </summary>
	public class PbArchs
	{
		/// <summary>
		/// Archive array<br/>
		/// Key: archive name<br/>
		/// Value: Archive object
		/// </summary>
		public Hashtable archives = null;
		public Hashtable internalArchs = new Hashtable();
		/// <summary>
		/// <p>List of commands used to inflate the archive.<br/>
		/// Key: extension<br/>
		/// Value: InstallCommand</p>
		/// <p>Command will be called with one parameter, the archive name.</p>
		/// <p>The list is restored from pbCmd.xml in restorePbCmd. inflate.pb in PHP version.</p>
		/// </summary>
		Hashtable inflate = new Hashtable();
		/// <summary>
		/// <p>Command used to remove an inflated archive. delete in PHP version.<br/>
		/// Key: extension<br/>
		/// Value: InstallCommand</p>
		/// <p>Command will be called with one parameter, the archive name.</p>
		/// <p>Restored from pbCmd.xml in restorePbCmd. inflate.pb in PHP version.</p>
		/// </summary>
		Hashtable remove = new Hashtable();
		/// <summary>
		/// Populates inflate and remove from pbCmd.xml.
		/// </summary>
		private void restorePbCmd()
		{
			if (!File.Exists(Global.path + "\\pbCmd.xml")) 
				return;
			XmlTextReader r = new XmlTextReader(Global.path + "\\pbCmd.xml");
			r.WhitespaceHandling = WhitespaceHandling.None;
			r.ReadStartElement("commands");
			restoreHash(inflate, r, "inflate");
			restoreHash(remove, r, "remove");
			r.Close();
		}
		/// <summary>
		/// Populate a Hashtable, either inflate or remove.
		/// </summary>
		/// <param name="h">Hashtable to populate</param>
		/// <param name="r">XML stream to read</param>
		/// <param name="elt">Element to process</param>
		private void restoreHash(Hashtable h, XmlReader r, string elt)
		{
			r.ReadStartElement(elt);
			while (true) 
			{
				if (r.NodeType == XmlNodeType.Element) 
				{
					if (r.LocalName.Equals("entry")) 
					{
						r.Read();
						string extension = r.ReadElementString("extension");
						string command = r.ReadElementString("command");
						string paramPrefix = r.ReadElementString("param-prefix");
						string useShell = r.ReadElementString("use-shell");
						InstallCommand ic;
						if (useShell.Equals("true"))
							ic = new InstallCommand(command, paramPrefix, true);
						else
							ic = new InstallCommand(command, paramPrefix, false);
						h.Add(extension, ic);
					} else
						return;
				}
				if ((r.NodeType == XmlNodeType.EndElement) && r.LocalName.Equals(elt)) 
					break;
				if (r.NodeType == XmlNodeType.None)
					return;
				r.Read();
			}
			r.ReadEndElement();
		}
		/// <summary>
		/// Creation of a new Archive list.
		/// </summary>
		public PbArchs()
		{
			archives = Hashtable.Synchronized(internalArchs);
			restorePbCmd();
		}
		/// <summary>
		/// Restores archives from pbArchives.xml.
		/// </summary>
		/// <param name="r">XML stream to restore</param>
		public PbArchs(XmlReader r)
		{
			restorePbCmd();
			r.ReadStartElement("PbArchs");
			r.ReadStartElement("archives");
			while (true) 
			{
				if (r.NodeType == XmlNodeType.Element) 
				{
					if (r.LocalName.Equals("entry")) 
					{
						r.Read();
						string arch = r.ReadElementString("arch");
						Archive a = new Archive(r);
						internalArchs.Add(arch, a);
					} 
					else 
					{
						archives = Hashtable.Synchronized(internalArchs);
						return;
					}
				}
				if ((r.NodeType == XmlNodeType.EndElement) && r.LocalName.Equals("archives")) 
					break;
				if (r.NodeType == XmlNodeType.None) 
				{
					archives = Hashtable.Synchronized(internalArchs);
					return;
				}
				r.Read();
			}
			archives = Hashtable.Synchronized(internalArchs);
		}
		/// <summary>
		/// Serializer.
		/// </summary>
		/// <param name="w">XML stream where the Repository archives must be serialized</param>
		public void WriteXml(XmlWriter w) 
		{
			w.WriteStartElement("PbArchs");
			w.WriteStartElement("archives");
			IDictionaryEnumerator ide = internalArchs.GetEnumerator();
			while(ide.MoveNext()) 
			{
				w.WriteStartElement("entry");
				w.WriteElementString("arch", (string)ide.Key);
				Archive a = (Archive)ide.Value;
				a.WriteXml(w);
				w.WriteEndElement();
			}
			w.WriteEndElement();
			w.WriteEndElement();
		}
		/// <summary>Deploy an archive.</summary>
		/// <returns>status</returns>
		/// <param name="a">archive</param>
		/// <param name="u">download URL</param>
		/// <param name="o">archive owner</param>
		/// <param name="s">archive size</param>
		/// <param name="t">archive' last modified time</param>
		/// <param name="d">download URL</param>
		public string add(string a, string u, string o, int s, DateTime t, string d) 
		{
			lock(this) 
			{
				int pos = a.LastIndexOf('.');
				if (pos == -1)
					return a + " without extension";
				string extension = a.Substring(pos);
				if (!inflate.Contains(extension))
					return a + " unsupported extension";
				Archive arch;
				if (archives.Contains(a)) 
				{
					arch = (Archive)archives[a];
					if (!arch.owner.Equals(o))
						return a + " invalid owner " + o;
					string duUp = arch.downloadURL.ToUpper();
					string uUp = u.ToUpper();
					if (!duUp.Equals(uUp))
						return a + " invalid repository";
					if (File.Exists(a))
						File.Delete(a);
					arch.size = s;
					arch.date = t;
					arch.docURL = d;
				} 
				else 
				{
					arch = new Archive(u, o, s, t, d);
					archives.Add(a, arch);
				}
				string downloadURL = u;
				if (u.IndexOf("://") == -1)
					downloadURL = "http://" + u;
				WebRequest WReq = WebRequest.Create(downloadURL + a);
				WebResponse WResp = WReq.GetResponse();
				Stream str = WResp.GetResponseStream();
				byte[] buf = new byte[s];
				int begin = 0;
				int l = str.Read(buf, begin, s);
				int len = s;
				while(l < len) 
				{
					if (l == -1) 
					{
						arch.state = "pending install";
						break;
					}
					begin += l;
					len -= l;
					if (len == 0) 
					{
						arch.state = "installed";
						break;
					}
					l = str.Read(buf, begin, len);
				}
				Global.savePbArchives();
				FileStream afs = File.Create(Global.path + "\\" + a);
				afs.Write(buf, 0, s);
				afs.Close();
				str.Close();
				InstallCommand ic = (InstallCommand)inflate[extension];
				if (ic.cmd.Equals("-"))
					return a + " installed";
				if (ic.cmd.IndexOf('\\') == -1)
					ic.cmd = Global.path + "\\" + ic.cmd;
				if (File.Exists(ic.cmd)) 
				{
					ProcessStartInfo psi = new ProcessStartInfo();
					psi.FileName = ic.cmd;
					// psi.RedirectStandardOutput;
					psi.UseShellExecute = ic.useShell;
					psi.Arguments = ic.paramPrefix + Global.path + "\\" + a;
					Process p = Process.Start(psi);
				}
				return a + " installed";
			}
		}
		/// <summary>Undeploy an archive</summary>
		/// <returns>status</returns>
		/// <param name="a">archive to remove</param>
		/// <param name="u">download URL</param>
		/// <param name="o">archive owner</param>
		public string delete(string a, string u, string o)
		{
			lock(this) 
			{
				if (!archives.Contains(a))
					return a + " is not installed";
				Archive arch = (Archive)archives[a];
				if (!arch.owner.Equals(o))
					return a + " invalid owner " + o;
				string duUp = arch.downloadURL.ToUpper();
				string uUp = u.ToUpper();
				if (!duUp.Equals(uUp))
					return a + " invalid repository";
				archives.Remove(a);
				Global.savePbArchives();
				int pos = a.LastIndexOf('.');
				if (pos != -1) 
				{
					string extension = a.Substring(pos);
					if (remove.Contains(extension)) 
					{
						InstallCommand ic = (InstallCommand)remove[extension];
						if (!ic.cmd.Equals("-"))
						{
							if (ic.cmd.IndexOf('\\') == -1)
								ic.cmd = Global.path + "\\" + ic.cmd;
							if (File.Exists(ic.cmd)) 
							{
								ProcessStartInfo psi = new ProcessStartInfo();
								psi.FileName = ic.cmd;
								// psi.RedirectStandardOutput;
								psi.UseShellExecute = ic.useShell;
								psi.Arguments = ic.paramPrefix + Global.path + "\\" + a;
								Process p = Process.Start(psi);
							}
						}
					}
				}
				if (File.Exists(Global.path + "\\" + a))
					File.Delete(Global.path + "\\" + a);
				return a + " uninstalled";
			}
		}
	}
	/// <summary>
	/// Describes a command.
	/// </summary>
	public sealed class InstallCommand 
	{
		/// <summary>
		/// Represents the command itself
		/// </summary>
		public string cmd;
		/// <summary>
		/// Parameter string before the archive parameter.
		/// For instance, /arch:
		/// </summary>
		public string paramPrefix;
		/// <summary>
		/// true if the command must run in a shell environment.
		/// For instance .bat.
		/// </summary>
		public bool useShell;
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="c">Command itself</param>
		/// <param name="p">Parameter string before archive name</param>
		/// <param name="u">Need a shell</param>
		public InstallCommand(string c, string p, bool u) 
		{
			cmd = c;
			paramPrefix = p;
			useShell = u;
		}
	}
	/// <summary>Describes a PageBox archive.</summary>
	public sealed class Archive
	{
		/// <summary>
		/// Download location (repository). download in PHP version.
		/// </summary>
		public string downloadURL;
		/// <summary>
		/// Archive owner (publisher)
		/// </summary>
		public string owner;
		/// <summary>
		/// Archive size
		/// </summary>
		public int size;
		/// <summary>
		/// Archive last modified time
		/// </summary>
		public DateTime date;
		/// <summary>
		/// Archive documentation URL. doc in PHP version
		/// </summary>
		public string docURL;
		/// <summary>
		/// Archive state
		/// </summary>
		public string state;
		/// <summary>Constructor.</summary>
		/// <param name="l">Download location</param>
		/// <param name="o">Archive owner</param>
		/// <param name="s">Archive size</param>
		/// <param name="t">Archive last modified time</param>
		/// <param name="d">Archive documentation URL</param>
		public Archive(string l, string o, int s, DateTime t, string d) 
		{
			downloadURL = l;
			owner = o;
			size = s;
			date = t;
			docURL = d;
		}
		/// <summary>Constructor and unserializer.</summary>
		/// <param name="r">XML stream to unserialize</param>
		public Archive(XmlReader r) 
		{
			r.ReadStartElement("Archive");
			downloadURL = r.ReadElementString("downloadURL");
			owner = r.ReadElementString("owner");
			size = XmlConvert.ToInt32(r.ReadElementString("size"));
			date = XmlConvert.ToDateTime(r.ReadElementString("date"));
			docURL = r.ReadElementString("docURL");
			r.ReadEndElement();
		}
		/// <summary>Serializer.</summary>
		/// <param name="w">stream where the Subscriber must be serialized</param>
		public void WriteXml(XmlWriter w)
		{
			w.WriteStartElement("Archive");
			w.WriteElementString("downloadURL", downloadURL);
			w.WriteElementString("owner", owner);
			w.WriteElementString("size", XmlConvert.ToString(size));
			w.WriteElementString("date", XmlConvert.ToString(date));
			w.WriteElementString("docURL", docURL);
			w.WriteEndElement();
		}
	}
}
